(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./Range", "../../Utils/NumberUtils"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.Circle = void 0;
    const Range_1 = require("./Range");
    const NumberUtils_1 = require("../../Utils/NumberUtils");
    class Circle extends Range_1.Range {
        constructor(x, y, radius) {
            super(x, y);
            this.radius = radius;
        }
        contains(point) {
            return (0, NumberUtils_1.getDistance)(point, this.position) <= this.radius;
        }
        intersects(range) {
            const rect = range, circle = range, pos1 = this.position, pos2 = range.position, xDist = Math.abs(pos2.x - pos1.x), yDist = Math.abs(pos2.y - pos1.y), r = this.radius;
            if (circle.radius !== undefined) {
                const rSum = r + circle.radius, dist = Math.sqrt(xDist * xDist + yDist + yDist);
                return rSum > dist;
            }
            else if (rect.size !== undefined) {
                const w = rect.size.width, h = rect.size.height, edges = Math.pow(xDist - w, 2) + Math.pow(yDist - h, 2);
                if (xDist > r + w || yDist > r + h) {
                    return false;
                }
                if (xDist <= w || yDist <= h) {
                    return true;
                }
                return edges <= r * r;
            }
            return false;
        }
    }
    exports.Circle = Circle;
});
